@@ -1,20 +1,25 @@
 #define _GNU_SOURCE /* For strcasestr */
 #include <string.h>
 
-#include "../common/misc/locales.h"
+#include "../helper/locales.h"
 
 #include <stdarg.h>
 #include <signal.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <gammu.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include "common.h"
 #include "misc.h"
-#include "formats.h"
 #include "message.h"
 
+#include "../helper/formats.h"
+#include "../helper/printing.h"
+#include "../helper/string.h"
+
 #ifdef GSM_ENABLE_NOKIA_DCT3
 #  include "depend/nokia/dct3.h"
 #  include "depend/nokia/dct3trac/wmx.h"
@@ -95,6 +100,82 @@ void PlayRingtone(int argc UNUSED, char *argv[])
 	GSM_Terminate();
 }
 
+/**
+ * Checks for firmware version available in Gammu database.
+ */
+void CheckFirmware(int argc UNUSED, char *argv[]UNUSED)
+{
+	GSM_File PhoneDB;
+	char model[GSM_MAX_MODEL_LENGTH];
+	char current_version[GSM_MAX_VERSION_LENGTH];
+	char url[70 + GSM_MAX_MODEL_LENGTH];
+	char latest_version[GSM_MAX_VERSION_LENGTH];
+	size_t pos = 0, oldpos = 0, i;
+	GSM_Error error;
+
+	GSM_Init(true);
+
+	/* Get model information */
+	error = GSM_GetModel(gsm, model);
+	Print_Error(error);
+
+	/* Empty string */
+	latest_version[0] = 0;
+
+	/* Request information from phone db */
+	sprintf(url, "http://www.gammu.org/support/phones/phonedbxml.php?model=%s", model);
+	PhoneDB.Buffer = NULL;
+	PhoneDB.Used = 0;
+	if (!GSM_ReadHTTPFile(url, &PhoneDB))
+		return;
+
+	/* Parse reply */
+	while (pos < PhoneDB.Used) {
+		if (PhoneDB.Buffer[pos] != 10) {
+			pos++;
+			continue;
+		}
+		PhoneDB.Buffer[pos] = 0;
+		if (strstr(PhoneDB.Buffer + oldpos, "<firmware>") == NULL) {
+			pos++;
+			oldpos = pos;
+			continue;
+		}
+		sprintf(latest_version, "%s", strstr(PhoneDB.Buffer + oldpos, "<version>") + 9);
+		for (i = 0; i < strlen(latest_version); i++) {
+			if (latest_version[i] == '<') {
+				latest_version[i] = 0;
+				break;
+			}
+		}
+		pos++;
+		oldpos = pos;
+	}
+	free(PhoneDB.Buffer);
+
+	/* Did we find something? */
+	if (latest_version[0] == 0) {
+		return;
+	}
+
+	/* Get phone firmware version */
+	error = GSM_GetFirmware(gsm, current_version, NULL, NULL);
+	Print_Error(error);
+
+	/* Compare versions */
+	if (!GSM_IsNewerVersion(latest_version, current_version))
+		return;
+
+	/* Print information to user */
+	printf_info(_("Never version of firmware is available!\n"));
+	printf_info(_("Latest version is %s and you run %s.\n"),
+			latest_version, current_version);
+
+	GSM_Terminate();
+}
+
+
+
 void Identify(int argc, char *argv[])
 {
 	char buffer[GSM_MAX_INFO_LENGTH];
@@ -189,7 +270,7 @@ void NetworkInfo(int argc UNUSED, char *argv[] UNUSED)
 	GSM_Terminate();
 }
 
-void IncomingCall(GSM_StateMachine *sm UNUSED, GSM_Call call)
+void IncomingCall(GSM_StateMachine *sm UNUSED, GSM_Call call, void *user_data)
 {
 	printf(LISTFORMAT, _("Call info"));
 	if (call.CallIDAvailable) printf(_("ID %i, "),call.CallID);
@@ -252,10 +333,10 @@ void Monitor(int argc, char *argv[])
 
 	GSM_Init(true);
 
-	GSM_SetIncomingSMSCallback(gsm, IncomingSMS);
-	GSM_SetIncomingCBCallback(gsm, IncomingCB);
-	GSM_SetIncomingCallCallback(gsm, IncomingCall);
-	GSM_SetIncomingUSSDCallback(gsm, IncomingUSSD);
+	GSM_SetIncomingSMSCallback(gsm, IncomingSMS, NULL);
+	GSM_SetIncomingCBCallback(gsm, IncomingCB, NULL);
+	GSM_SetIncomingCallCallback(gsm, IncomingCall, NULL);
+	GSM_SetIncomingUSSDCallback(gsm, IncomingUSSD, NULL);
 
 	error=GSM_SetIncomingSMS  		(gsm,true);
 	printf("%-35s : %s\n", _("Enabling info about incoming SMS"), GSM_ErrorString(error));
@@ -281,7 +362,7 @@ void Monitor(int argc, char *argv[])
 		CHECKMEMORYSTATUS(MemStatus,MEM_ME,"Phone phonebook");
 		CHECK_EXIT;
 		if ( (error = GSM_GetToDoStatus(gsm, &ToDoStatus)) == ERR_NONE) {
-			PRINTUSED(_("ToDos"), ToDoStatus.Used, ToDoStatus.Free);
+			PRINTUSED(_("Todos"), ToDoStatus.Used, ToDoStatus.Free);
 		}
 		CHECK_EXIT;
 		if ( (error = GSM_GetCalendarStatus(gsm, &CalendarStatus)) == ERR_NONE) {
@@ -505,7 +586,7 @@ void DialVoice(int argc, char *argv[])
 		} else if (strcasecmp(argv[3],"hide") == 0) {	ShowNumber = GSM_CALL_HideNumber;
 		} else {
 			printf(_("Unknown parameter (\"%s\")\n"),argv[3]);
-			exit(-1);
+			Terminate(3);
 		}
 	}
 
@@ -519,7 +600,7 @@ void DialVoice(int argc, char *argv[])
 
 int TerminateID = -1;
 
-void IncomingCall0(GSM_StateMachine *sm UNUSED, GSM_Call call)
+void IncomingCall0(GSM_StateMachine *sm UNUSED, GSM_Call call, void * user_data)
 {
 	if (call.CallIDAvailable) TerminateID = call.CallID;
 }
@@ -534,14 +615,14 @@ void MakeTerminatedCall(int argc, char *argv[])
 		} else if (strcasecmp(argv[4],"hide") == 0) {	ShowNumber = GSM_CALL_HideNumber;
 		} else {
 			printf(_("Unknown parameter (\"%s\")\n"),argv[4]);
-			exit(-1);
+			Terminate(3);
 		}
 	}
 
 	GSM_Init(true);
 
 	TerminateID = -1;
-	GSM_SetIncomingCallCallback(gsm, IncomingCall0);
+	GSM_SetIncomingCallCallback(gsm, IncomingCall0, NULL);
 
 	error=GSM_SetIncomingCall(gsm,true);
 	Print_Error(error);
@@ -674,7 +755,7 @@ void Reset(int argc UNUSED, char *argv[])
 	} else if (strcasecmp(argv[2],"HARD") == 0) {	hard=true;
 	} else {
 		printf(_("What type of reset do you want (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	GSM_Init(true);
@@ -785,7 +866,7 @@ void GetBitmap(int argc, char *argv[])
 		MultiBitmap.Bitmap[0].Type=GSM_OperatorLogo;
 	} else {
 		printf(_("What type of logo do you want to get (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 	MultiBitmap.Bitmap[0].Location=location;
 
@@ -801,7 +882,7 @@ void GetBitmap(int argc, char *argv[])
 	case GSM_CallerGroupLogo:
 		if (!MultiBitmap.Bitmap[0].DefaultBitmap) GSM_PrintBitmap(stdout,&MultiBitmap.Bitmap[0]);
 		printf(LISTFORMAT "\"%s\"", _("Group name"),DecodeUnicodeConsole(MultiBitmap.Bitmap[0].Text));
-		if (MultiBitmap.Bitmap[0].DefaultName) printf(_(" (default)"));
+		if (MultiBitmap.Bitmap[0].DefaultName) printf(" (%s)", _("default"));
 		printf("\n");
 		if (MultiBitmap.Bitmap[0].DefaultRingtone) {
 			printf(LISTFORMAT "%s\n", _("Ringtone"), _("default"));
@@ -828,10 +909,12 @@ void GetBitmap(int argc, char *argv[])
 
 			printf(LISTFORMAT, _("Ringtone"));
 			if (UnicodeLength(GSM_GetRingtoneName(&Info,MultiBitmap.Bitmap[0].RingtoneID))!=0) {
+				/* l10n: Ringtone name and ID format */
 				printf(_("\"%s\" (ID %i)\n"),
 					DecodeUnicodeConsole(GSM_GetRingtoneName(&Info,MultiBitmap.Bitmap[0].RingtoneID)),
 					MultiBitmap.Bitmap[0].RingtoneID);
 			} else {
+				/* l10n: Ringtone ID format */
 				printf(_("ID %i\n"),MultiBitmap.Bitmap[0].RingtoneID);
 			}
 
@@ -893,7 +976,7 @@ void SetBitmap(int argc, char *argv[])
 	if (strcasecmp(argv[2],"STARTUP") == 0) {
 		if (argc<4) {
 			printf("%s\n", _("More parameters required!"));
-			exit(-1);
+			Terminate(3);
 		}
 		MultiBitmap.Bitmap[0].Type=GSM_StartupLogo;
 		MultiBitmap.Bitmap[0].Location=1;
@@ -908,21 +991,21 @@ void SetBitmap(int argc, char *argv[])
 	} else if (strcasecmp(argv[2],"TEXT") == 0) {
 		if (argc<4) {
 			printf("%s\n", _("More parameters required!"));
-			exit(-1);
+			Terminate(3);
 		}
 		Bitmap.Type=GSM_WelcomeNote_Text;
 		EncodeUnicode(Bitmap.Text,argv[3],strlen(argv[3]));
 	} else if (strcasecmp(argv[2],"DEALER") == 0) {
 		if (argc<4) {
 			printf("%s\n", _("More parameters required!"));
-			exit(-1);
+			Terminate(3);
 		}
 		Bitmap.Type=GSM_DealerNote_Text;
 		EncodeUnicode(Bitmap.Text,argv[3],strlen(argv[3]));
 	} else if (strcasecmp(argv[2],"CALLER") == 0) {
 		if (argc<4) {
 			printf("%s\n", _("More parameters required!"));
-			exit(-1);
+			Terminate(3);
 		}
 		GetStartStop(&i, NULL, 3, argc, argv);
 		if (i>5 && i!=255) {
@@ -952,7 +1035,7 @@ void SetBitmap(int argc, char *argv[])
 	} else if (strcasecmp(argv[2],"PICTURE") == 0) {
 		if (argc<5) {
 			printf("%s\n", _("More parameters required!"));
-			exit(-1);
+			Terminate(3);
 		}
 		MultiBitmap.Bitmap[0].Type		= GSM_PictureImage;
 		MultiBitmap.Bitmap[0].Location		= atoi(argv[4]);
@@ -1010,7 +1093,7 @@ void SetBitmap(int argc, char *argv[])
 		memcpy(&Bitmap,&MultiBitmap.Bitmap[0],sizeof(GSM_Bitmap));
 	} else {
 		printf(_("What type of logo do you want to set (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	if (init) GSM_Init(true);
@@ -1051,7 +1134,7 @@ void SetRingtone(int argc, char *argv[])
 				break;
 			}
 			printf(_("Unknown parameter (\"%s\")"),argv[i]);
-			exit(-1);
+			Terminate(3);
 		case 1:
 			ringtone.Location=atoi(argv[i]);
 			nextlong = 0;
@@ -1064,7 +1147,7 @@ void SetRingtone(int argc, char *argv[])
 	}
 	if (nextlong!=0) {
 		printf_err("%s\n", _("Parameter missing!"));
-		exit(-1);
+		Terminate(3);
 	}
 	if (ringtone.Location==0) {
 		printf_err("%s\n", _("Please enumerate locations from 1"));
@@ -1103,11 +1186,13 @@ void ClearMemory(GSM_MemoryType type, const char *question)
 				Pbk.EntriesNum	= 0;
 				error=GSM_DeleteMemory(gsm, &Pbk);
 				Print_Error(error);
-				fprintf(stderr, _("%cClearing: %i percent"), 13,
-						(i + 1) * 100 / (MemStatus.MemoryUsed + MemStatus.MemoryFree));
+				fprintf(stderr, "\r");
+				fprintf(stderr, "%s ", _("Deleting:"));
+				fprintf(stderr, _("%i percent"),
+					(i + 1) * 100 / (MemStatus.MemoryUsed + MemStatus.MemoryFree));
 				if (gshutdown) {
 					GSM_Terminate();
-					exit(0);
+					Terminate(0);
 				}
 			}
 			fprintf(stderr, "\n");
@@ -1155,6 +1240,7 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 				fprintf(stderr, "*");
 			}
 			fprintf(stderr, "\n");
+			fflush(stderr);
 		} else {
 			fprintf(stderr, "%s\n", _("Done"));
 			Print_Error(error);
@@ -1164,7 +1250,7 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 	DoClear = false;
 	error = GSM_GetToDoStatus(gsm,&ToDoStatus);
 	if (error == ERR_NONE && ToDoStatus.Used != 0) {
-		if (answer_yes(_("Delete phone ToDo?"))) DoClear = true;
+		if (answer_yes(_("Delete phone todos?"))) DoClear = true;
 	}
 	if (DoClear) {
 		fprintf(stderr, LISTFORMAT, _("Deleting"));
@@ -1178,6 +1264,7 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 				fprintf(stderr, "*");
 			}
 			fprintf(stderr, "\n");
+			fflush(stderr);
 		} else {
 			fprintf(stderr, "%s\n", _("Done"));
 			Print_Error(error);
@@ -1187,7 +1274,7 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 	DoClear = false;
 	error = GSM_GetNotesStatus(gsm,&ToDoStatus);
 	if (error == ERR_NONE && ToDoStatus.Used != 0) {
-		if (answer_yes(_("Delete phone Notes?"))) DoClear = true;
+		if (answer_yes(_("Delete phone notes?"))) DoClear = true;
 	}
 	if (DoClear) {
 		fprintf(stderr, LISTFORMAT, _("Deleting"));
@@ -1199,6 +1286,7 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 			fprintf(stderr, "*");
 		}
 		fprintf(stderr, "\n");
+		fflush(stderr);
 	}
 
 	Bookmark.Location = 1;
@@ -1219,6 +1307,7 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 				fprintf(stderr, "*");
 			}
 			fprintf(stderr, "\n");
+			fflush(stderr);
 		}
 	}
 	if (answer_yes(_("Delete all phone user ringtones?"))) {
@@ -1242,17 +1331,21 @@ void ClearAll(int argc UNUSED, char *argv[] UNUSED)
 void DisplayConnectionSettings(GSM_MultiWAPSettings *settings,int j)
 {
 	if (settings->Settings[j].IsContinuous) {
-		printf(_("Connection type     : Continuous\n"));
+		printf(LISTFORMAT "%s\n", _("Connection type"), _("Continuous"));
 	} else {
-		printf(_("Connection type     : Temporary\n"));
+		printf(LISTFORMAT "%s\n", _("Connection type"), _("Temporary"));
 	}
 	if (settings->Settings[j].IsSecurity) {
-		printf(_("Connection security : On\n"));
+		printf(LISTFORMAT "%s\n", _("Connection security"), _("On"));
 	} else {
-		printf(_("Connection security : Off\n"));
+		printf(LISTFORMAT "%s\n", _("Connection security"), _("Off"));
 	}
-	printf(_("Proxy               : address \"%s\", port %i\n"),DecodeUnicodeConsole(settings->Proxy),settings->ProxyPort);
-	printf(_("2'nd proxy          : address \"%s\", port %i\n"),DecodeUnicodeConsole(settings->Proxy2),settings->Proxy2Port);
+	printf(LISTFORMAT, _("Proxy"));
+	printf(_("address \"%s\", port %i"), DecodeUnicodeConsole(settings->Proxy), settings->ProxyPort);
+	printf("\n");
+	printf(LISTFORMAT, _("Second proxy"));
+	printf(_("address \"%s\", port %i"), DecodeUnicodeConsole(settings->Proxy2), settings->Proxy2Port);
+	printf("\n");
 	switch (settings->Settings[j].Bearer) {
 	case WAPSETTINGS_BEARER_SMS:
 		printf(LISTFORMAT "%s", _("Bearer"), _("SMS"));
@@ -1268,24 +1361,30 @@ void DisplayConnectionSettings(GSM_MultiWAPSettings *settings,int j)
 		printf(LISTFORMAT "\"%s\"\n", _("Dial-up number"),DecodeUnicodeConsole(settings->Settings[j].DialUp));
 		printf(LISTFORMAT "\"%s\"\n", _("IP address"),DecodeUnicodeConsole(settings->Settings[j].IPAddress));
 		if (settings->Settings[j].ManualLogin) {
-			printf(_("Login Type          : Manual\n"));
+			printf(LISTFORMAT "%s\n", _("Login type"), _("Manual"));
 		} else {
-			printf(_("Login Type          : Automatic\n"));
+			printf(LISTFORMAT "%s\n", _("Login type"), _("Automatic"));
 		}
 		if (settings->Settings[j].IsNormalAuthentication) {
-			printf(_("Authentication type : Normal\n"));
+			printf(LISTFORMAT "%s\n", _("Authentication type"), _("Normal"));
 		} else {
-			printf(_("Authentication type : Secure\n"));
+			printf(LISTFORMAT "%s\n", _("Authentication type"), _("Secure"));
 		}
 		if (settings->Settings[j].IsISDNCall) {
-			printf(_("Data call type      : ISDN\n"));
+			printf(LISTFORMAT "%s\n", _("Data call type"), _("ISDN"));
               	} else {
-			printf(_("Data call type      : Analogue\n"));
+			printf(LISTFORMAT "%s\n", _("Data call type"), _("Analogue"));
 		}
 		switch (settings->Settings[j].Speed) {
-			case WAPSETTINGS_SPEED_9600  : printf(_("Data call speed     : 9600\n"));  break;
-			case WAPSETTINGS_SPEED_14400 : printf(_("Data call speed     : 14400\n")); break;
-			case WAPSETTINGS_SPEED_AUTO  : printf(_("Data call speed     : Auto\n"));  break;
+			case WAPSETTINGS_SPEED_9600  :
+				printf(LISTFORMAT "%s\n", _("Data call speed"), "9600");
+				break;
+			case WAPSETTINGS_SPEED_14400 :
+				printf(LISTFORMAT "%s\n", _("Data call speed"), "14400");
+				break;
+			case WAPSETTINGS_SPEED_AUTO  :
+				printf(LISTFORMAT "%s\n", _("Data call speed"), _("Auto"));
+				break;
 		}
 		printf(LISTFORMAT "\"%s\"\n", _("User name"),DecodeUnicodeConsole(settings->Settings[j].User));
 		printf(LISTFORMAT "\"%s\"\n", _("Password"),DecodeUnicodeConsole(settings->Settings[j].Password));
@@ -1296,9 +1395,11 @@ void DisplayConnectionSettings(GSM_MultiWAPSettings *settings,int j)
 		printf("\n");
 		printf(LISTFORMAT "\"%s\"\n", _("Service code"),DecodeUnicodeConsole(settings->Settings[j].Code));
 		if (settings->Settings[j].IsIP) {
-			printf(_("Address type        : IP address\nIPaddress           : \"%s\"\n"),DecodeUnicodeConsole(settings->Settings[j].Service));
+			printf(LISTFORMAT "%s\n", _("Address type"), _("IP address"));
+			printf(LISTFORMAT "%s\n", _("IP address"), DecodeUnicodeConsole(settings->Settings[j].Service));
 		} else {
-			printf(_("Address type        : Service number\nService number      : \"%s\"\n"),DecodeUnicodeConsole(settings->Settings[j].Service));
+			printf(LISTFORMAT "%s\n", _("Address type"), _("Service number"));
+			printf(LISTFORMAT "%s\n", _("Service number"), DecodeUnicodeConsole(settings->Settings[j].Service));
 		}
 		break;
 	case WAPSETTINGS_BEARER_GPRS:
@@ -1306,14 +1407,14 @@ void DisplayConnectionSettings(GSM_MultiWAPSettings *settings,int j)
 		if (settings->ActiveBearer == WAPSETTINGS_BEARER_GPRS) printf(_(" (active)"));
 		printf("\n");
 		if (settings->Settings[j].ManualLogin) {
-			printf(_("Login Type          : Manual\n"));
+			printf(LISTFORMAT "%s\n", _("Login type"), _("Manual"));
 		} else {
-			printf(_("Login Type          : Automatic\n"));
+			printf(LISTFORMAT "%s\n", _("Login type"), _("Automatic"));
 		}
 		if (settings->Settings[j].IsNormalAuthentication) {
-			printf(_("Authentication type : Normal\n"));
+			printf(LISTFORMAT "%s\n", _("Authentication type"), _("Normal"));
 		} else {
-			printf(_("Authentication type : Secure\n"));
+			printf(LISTFORMAT "%s\n", _("Authentication type"), _("Secure"));
 		}
 		printf(LISTFORMAT "\"%s\"\n", _("Access point"),DecodeUnicodeConsole(settings->Settings[j].DialUp));
 		printf(LISTFORMAT "\"%s\"\n", _("IP address"),DecodeUnicodeConsole(settings->Settings[j].IPAddress));
@@ -1398,10 +1499,11 @@ void GetChatSettings(int argc, char *argv[])
 		printf(LISTFORMAT "\"%s\"\n", _("Password"),DecodeUnicodeConsole(settings.Password));
 		printf("\n");
 		for (j=0;j<settings.Connection.Number;j++) {
+			printf(LISTFORMAT, _("Connection set name"));
 			if (settings.Connection.Settings[j].Title[0]==0 && settings.Connection.Settings[j].Title[1]==0) {
-				printf(_("Connection set name : Set %i\n"),i);
+				printf(_("Set %i"),i);
 			} else {
-				printf(_("Connection set name : %s\n"),DecodeUnicodeConsole(settings.Connection.Settings[j].Title));
+				printf("%s",DecodeUnicodeConsole(settings.Connection.Settings[j].Title));
 			}
 			DisplayConnectionSettings(&settings.Connection,j);
 			printf("\n");
@@ -1435,9 +1537,15 @@ void GetWAPMMSSettings(int argc, char *argv[])
 			} else {
 				printf("%s", DecodeUnicodeConsole(settings.Settings[j].Title));
 			}
-			if (settings.Active) printf(_(" (active)"));
-			if (settings.ReadOnly) printf(_("\nRead only           : yes"));
-			printf(LISTFORMAT "\"%s\"\n", _("\nHomepage"),DecodeUnicodeConsole(settings.Settings[j].HomePage));
+			if (settings.Active) {
+				printf(" (%s)", _("active"));
+			}
+			printf("\n");
+			if (settings.ReadOnly) {
+				printf(LISTFORMAT "%s\n", _("Read only"), _("yes"));
+			}
+			printf(LISTFORMAT "\"%s\"\n", _("Homepage"),
+				DecodeUnicodeConsole(settings.Settings[j].HomePage));
 			DisplayConnectionSettings(&settings,j);
 			printf("\n");
 		}
@@ -1465,7 +1573,9 @@ void CopyBitmap(int argc, char *argv[])
 				case GSM_CallerGroupLogo  :	printf(_("Caller group logo")); 	break;
 				default		     : 					break;
 			}
-			printf(_(", width %i, height %i\n"),Bitmap.Bitmap[i].BitmapWidth,Bitmap.Bitmap[i].BitmapHeight);
+			printf(_(", width %i, height %i\n"),
+				(int)Bitmap.Bitmap[i].BitmapWidth,
+				(int)Bitmap.Bitmap[i].BitmapHeight);
 			GSM_PrintBitmap(stdout,&Bitmap.Bitmap[i]);
 		}
 	} else {
@@ -1481,7 +1591,7 @@ void CopyBitmap(int argc, char *argv[])
 					Bitmap.Bitmap[i].Type = GSM_OperatorLogo;
 				} else {
 					printf(_("What format of output file logo (\"%s\") ?\n"),argv[4]);
-					exit(-1);
+					Terminate(3);
 				}
 			}
 		}
@@ -1506,7 +1616,7 @@ void CopyRingtone(int argc, char *argv[])
 		} else if (strcasecmp(argv[4],"BINARY") == 0) {	Format = RING_NOKIABINARY;
 		} else {
 			printf(_("What format of output ringtone file (\"%s\") ?\n"),argv[4]);
-			exit(-1);
+			Terminate(3);
 		}
 	}
 
@@ -1526,7 +1636,7 @@ void PressKeySequence(int argc UNUSED, char *argv[])
 	error = MakeKeySequence(argv[2], KeyCode, &Length);
 	if (error == ERR_NOTSUPPORTED) {
 		printf(_("Unknown key/function name: \"%c\"\n"),argv[2][Length]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	GSM_Init(true);
@@ -1556,7 +1666,7 @@ void GetAllCategories(int argc UNUSED, char *argv[])
         	Status.Type 	= Category_Phonebook;
 	} else {
 		printf(_("What type of categories do you want to get (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	GSM_Init(true);
@@ -1594,7 +1704,7 @@ void GetCategory(int argc, char *argv[])
     		Category.Type = Category_Phonebook;
 	} else {
 		printf(_("What type of categories do you want to get (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	GetStartStop(&start, &stop, 2, argc - 1, argv + 1);
@@ -1632,7 +1742,7 @@ void AddCategory(int argc UNUSED, char *argv[])
     		Category.Type = Category_Phonebook;
 	} else {
 		printf(_("What type of category do you want to add (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	GSM_Init(true);
@@ -1666,25 +1776,39 @@ void EnterSecurityCode(int argc UNUSED, char *argv[])
 {
 	GSM_SecurityCode Code;
 	GSM_Error error;
+	char message[200];
+#ifdef HAVE_GETPASS
+	char *pass;
+#endif
 
 	if (strcasecmp(argv[2],"PIN") == 0) {		Code.Type = SEC_Pin;
 	} else if (strcasecmp(argv[2],"PUK") == 0) {	Code.Type = SEC_Puk;
 	} else if (strcasecmp(argv[2],"PIN2") == 0) {	Code.Type = SEC_Pin2;
 	} else if (strcasecmp(argv[2],"PUK2") == 0) {	Code.Type = SEC_Puk2;
+	} else if (strcasecmp(argv[2],"PHONE") == 0) {	Code.Type = SEC_Phone;
+	} else if (strcasecmp(argv[2],"NETWORK") == 0) {	Code.Type = SEC_Network;
 	} else {
 		printf_err("%s: %s\n", _("Invalid security code type"), argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	if (strcmp(argv[3], "-") == 0) {
+		sprintf(message, _("Enter %s code: "), argv[2]);
+#ifdef HAVE_GETPASS
+		pass = getpass(message);
+		strcpy(Code.Code, pass);
+#else
 		/* Read code from stdin */
+#ifdef HAVE_UNISTD_H
 		if (isatty(fileno(stdin))) {
-			printf(_("Enter %s code: "), argv[2]);
+			printf("%s", message);
 		}
+#endif
 		if (fscanf(stdin, "%15s", Code.Code) != 1) {
 			printf_err("%s\n", _("No PIN code entered!"));
-			exit(-1);
+			Terminate(3);
 		}
+#endif
 	} else {
 		strcpy(Code.Code,argv[3]);
 	}
@@ -1885,7 +2009,7 @@ void ResetPhoneSettings(int argc UNUSED, char *argv[])
 	} else if (strcasecmp(argv[2],"FACTORY") == 0) { Type = GSM_RESET_FULLFACTORY;
 	} else {
 		printf(_("What type of reset phone settings (\"%s\") ?\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	GSM_Init(true);
@@ -1985,7 +2109,7 @@ void CallDivert(int argc, char *argv[])
 	else if (strcasecmp("set",	argv[2]) == 0) {}
 	else {
 		printf(_("Unknown divert action (\"%s\")\n"),argv[2]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	     if (strcasecmp("all"       , argv[3]) == 0) {cd.Request.DivertType = GSM_DIVERT_AllTypes  ;}
@@ -1994,7 +2118,7 @@ void CallDivert(int argc, char *argv[])
 	else if (strcasecmp("outofreach", argv[3]) == 0) {cd.Request.DivertType = GSM_DIVERT_OutOfReach;}
 	else {
 		printf(_("Unknown divert type (\"%s\")\n"),argv[3]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	     if (strcasecmp("all"  , argv[4]) == 0) {cd.Request.CallType = GSM_DIVERT_AllCalls  ;}
@@ -2003,7 +2127,7 @@ void CallDivert(int argc, char *argv[])
 	else if (strcasecmp("data" , argv[4]) == 0) {cd.Request.CallType = GSM_DIVERT_DataCalls ;}
 	else {
 		printf(_("Unknown call type (\"%s\")\n"),argv[4]);
-		exit(-1);
+		Terminate(3);
 	}
 
 	GSM_Init(true);
@@ -2011,7 +2135,9 @@ void CallDivert(int argc, char *argv[])
 	if (strcasecmp("get", argv[2]) == 0) {
 		error = GSM_GetCallDivert(gsm,&cd);
 	    	Print_Error(error);
-		printf(_("Query:\n   Divert type: "));
+		printf(_("Query:"));
+		printf("\n");
+		printf("  " LISTFORMAT, _("Divert type"));
 	} else {
 		cd.Request.Number[0] = 0;
 		cd.Request.Number[1] = 0;
@@ -2022,7 +2148,9 @@ void CallDivert(int argc, char *argv[])
 
 		error = GSM_SetCallDivert(gsm,&cd);
 	    	Print_Error(error);
-		printf(_("Changed:\n   Divert type: "));
+		printf(_("Changed:"));
+		printf("\n");
+		printf("  " LISTFORMAT, _("Divert type"));
 	}
 
       	switch (cd.Request.DivertType) {
@@ -2032,7 +2160,8 @@ void CallDivert(int argc, char *argv[])
                 case GSM_DIVERT_AllTypes  : printf(_("all types of diverts"));			break;
                 default		          : printf(_("unknown %i"),cd.Request.DivertType);			break;
         }
-        printf(_("\n   Calls type : "));
+	printf("\n");
+	printf("  " LISTFORMAT, _("Call type"));
 	switch (cd.Request.CallType) {
 	 	case GSM_DIVERT_VoiceCalls: printf(_("voice"));				break;
                 case GSM_DIVERT_FaxCalls  : printf(_("fax"));				break;
@@ -2040,10 +2169,13 @@ void CallDivert(int argc, char *argv[])
 		case GSM_DIVERT_AllCalls  : printf(_("data & fax & voice"));		break;
                 default                   : printf(_("unknown %i"),cd.Request.CallType);   		break;
         }
-	printf(_("\nResponse:"));
+	printf("\n");
+	printf(_("Response:"));
+	printf("\n");
 
 	for (i=0;i<cd.Response.EntriesNum;i++) {
-	        printf(_("\n   Calls type : "));
+		printf("\n");
+		printf("  " LISTFORMAT, _("Call type"));
         	switch (cd.Response.Entries[i].CallType) {
                 	case GSM_DIVERT_VoiceCalls: printf(_("voice"));		 	break;
                 	case GSM_DIVERT_FaxCalls  : printf(_("fax"));		 	break;
@@ -2051,10 +2183,10 @@ void CallDivert(int argc, char *argv[])
                 	default                   : printf(_("unknown %i"),cd.Response.Entries[i].CallType);break;
               	}
 		printf("\n");
-               	printf(_("   Timeout    : "));
+		printf("  " LISTFORMAT, _("Timeout"));
 		PRINTSECONDS(cd.Response.Entries[i].Timeout);
 		printf("\n");
-                printf(LISTFORMAT "%s\n", _("Number"),DecodeUnicodeString(cd.Response.Entries[i].Number));
+                printf("  "LISTFORMAT "%s\n", _("Number"),DecodeUnicodeString(cd.Response.Entries[i].Number));
         }
 	printf("\n");
 
