@@ -140,7 +140,7 @@ typedef enum {
 	/**
 	 * Power failure
 	 */
-	GSM_PowerFault,
+	GSM_PowerFault
 } GSM_ChargeState;
 
 /**
@@ -164,7 +164,7 @@ typedef enum {
 	/**
 	 * Lithium Polymer battery
 	 */
-	GSM_BatteryLiPol,
+	GSM_BatteryLiPol
 } GSM_BatteryType;
 
 /**
@@ -248,8 +248,6 @@ typedef struct {
  * be determined on run time.
  *
  * \ingroup Info
- *
- * \todo API change: should contain GSM prefix.
  */
 typedef enum {
 	/* n6110.c */
@@ -577,81 +575,79 @@ typedef enum {
 	/**
 	 * Just marker of highest feature code, should not be used.
 	 */
-	F_LAST_VALUE,
-} Feature;
+	F_LAST_VALUE
+} GSM_Feature;
 
 /**
  * Converts feature value to string.
  *
  * \ingroup Info
  *
- * \param feature Feature to convert.
+ * \param feature GSM_Feature to convert.
  *
  * \return Pointer to static string with string for specified feature,
  * NULL on failure.
  */
-const char *GSM_FeatureToString(Feature feature);
+const char *GSM_FeatureToString(GSM_Feature feature);
 
 /**
  * Converts feature string to value.
  *
  * \ingroup Info
  *
- * \param feature Feature string to convert.
+ * \param feature GSM_Feature string to convert.
  *
- * \return Feature value, 0 on failure.
+ * \return GSM_Feature value, 0 on failure.
  */
-Feature GSM_FeatureFromString(const char *feature);
+GSM_Feature GSM_FeatureFromString(const char *feature);
 
 /**
  * Model identification, used for finding phone features.
  *
  * \ingroup Info
- *
- * \todo API change: should contain GSM prefix.
  */
 typedef struct {
 	/**
 	 * Model as returned by phone
 	 */
-	char *model;
+	const char *model;
 	/**
 	 * Identification by Gammu
 	 */
-	char *number;
+	const char *number;
 	/**
 	 * Model as used over IrDA
 	 */
-	char *irdamodel;
+	const char *irdamodel;
 	/**
 	 * List of supported features
 	 */
-	Feature features[GSM_MAX_PHONE_FEATURES + 1];
-} OnePhoneModel;
+	GSM_Feature features[GSM_MAX_PHONE_FEATURES + 1];
+} GSM_PhoneModel;
 
 /**
  * Checks whether phone supports features.
  *
  * \param model Model information (you can get it using \ref GSM_GetModelInfo).
- * \param feature Feature to check for.
+ * \param feature GSM_Feature to check for.
  *
  * \return True if phone has defined this feature.
  *
  * \ingroup Info
  */
-bool GSM_IsPhoneFeatureAvailable(OnePhoneModel * model, Feature feature);
+bool GSM_IsPhoneFeatureAvailable(GSM_PhoneModel * model, GSM_Feature feature);
 
 /**
  * Adds feature to phone configuration.
  *
  * \param model Model information (you can get it using \ref GSM_GetModelInfo).
- * \param feature Feature to check for.
+ * \param feature GSM_Feature to check for.
  *
  * \return True if phone has defined this feature.
  *
  * \ingroup Info
  */
-bool GSM_AddPhoneFeature(OnePhoneModel * model, Feature feature);
+bool GSM_AddPhoneFeature(GSM_PhoneModel * model, GSM_Feature feature);
 
 #include <gammu-statemachine.h>
 
@@ -674,7 +670,7 @@ GSM_Error GSM_GetModel(GSM_StateMachine * s, char *value);
  *
  * \ingroup Info
  */
-OnePhoneModel *GSM_GetModelInfo(GSM_StateMachine * s);
+GSM_PhoneModel *GSM_GetModelInfo(GSM_StateMachine * s);
 
 /**
  * Reads firmware information from phone.
