@@ -15,6 +15,7 @@
 #include <gammu-wap.h>
 #include <gammu-calendar.h>
 #include <gammu-file.h>
+#include <gammu-debug.h>
 
 /**
  * \defgroup Message Messages
@@ -548,30 +549,32 @@ typedef struct {
  *
  * \ingroup SMS
  */
-GSM_Error GSM_DecodeSMSFrame(GSM_SMSMessage * SMS, unsigned char *buffer,
+GSM_Error GSM_DecodeSMSFrame(GSM_Debug_Info * di, GSM_SMSMessage * SMS,
+			     unsigned char *buffer,
 			     GSM_SMSMessageLayout Layout);
 
 /**
  * Finds out coding type based on TPDCS header byte as defined by GSM
  * 03.38.
  */
-GSM_Coding_Type GSM_GetMessageCoding(const char TPDCS);
+GSM_Coding_Type GSM_GetMessageCoding(GSM_Debug_Info * di, const char TPDCS);
 
 /**
  * Encodes SMS frame.
  *
  * \ingroup SMS
  */
-GSM_Error GSM_EncodeSMSFrame(GSM_SMSMessage * SMS, unsigned char *buffer,
-			     GSM_SMSMessageLayout Layout, int *length,
-			     bool clear);
+GSM_Error GSM_EncodeSMSFrame(GSM_Debug_Info * di, GSM_SMSMessage * SMS,
+			     unsigned char *buffer, GSM_SMSMessageLayout Layout,
+			     int *length, bool clear);
 
 /**
  * Decodes SMS frame for status report.
  *
  * \ingroup SMS
  */
-GSM_Error GSM_DecodeSMSFrameStatusReportData(GSM_SMSMessage * SMS,
+GSM_Error GSM_DecodeSMSFrameStatusReportData(GSM_Debug_Info * di,
+					     GSM_SMSMessage * SMS,
 					     unsigned char *buffer,
 					     GSM_SMSMessageLayout Layout);
 
@@ -580,7 +583,8 @@ GSM_Error GSM_DecodeSMSFrameStatusReportData(GSM_SMSMessage * SMS,
  *
  * \ingroup SMS
  */
-GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage * SMS, unsigned char *buffer,
+GSM_Error GSM_DecodeSMSFrameText(GSM_Debug_Info * di, GSM_SMSMessage * SMS,
+				 unsigned char *buffer,
 				 GSM_SMSMessageLayout Layout);
 
 /**
@@ -588,14 +592,14 @@ GSM_Error GSM_DecodeSMSFrameText(GSM_SMSMessage * SMS, unsigned char *buffer,
  *
  * \ingroup SMS
  */
-void GSM_DecodeUDHHeader(GSM_UDHHeader * UDH);
+void GSM_DecodeUDHHeader(GSM_Debug_Info * di, GSM_UDHHeader * UDH);
 
 /**
  * Encodes UDH header.
  *
  * \ingroup SMS
  */
-void GSM_EncodeUDHHeader(GSM_UDHHeader * UDH);
+void GSM_EncodeUDHHeader(GSM_Debug_Info * di, GSM_UDHHeader * UDH);
 
 /**
  * Sets default content for SMS except for changing locations.
@@ -607,7 +611,6 @@ void GSM_EncodeUDHHeader(GSM_UDHHeader * UDH);
  */
 void GSM_SetDefaultReceivedSMSData(GSM_SMSMessage * SMS);
 
-
 /**
  * Sets default content for SMS. Use this for clearing structure.
  *
@@ -679,7 +682,8 @@ typedef struct {
  *
  * \ingroup SMS
  */
-bool GSM_DecodeSiemensOTASMS(GSM_SiemensOTASMSInfo * Info,
+bool GSM_DecodeSiemensOTASMS(GSM_Debug_Info * di,
+			     GSM_SiemensOTASMSInfo * Info,
 			     GSM_SMSMessage * SMS);
 
 /**
@@ -955,7 +959,8 @@ GSM_Error PHONE_EncodeSMSFrame(GSM_StateMachine * s, GSM_SMSMessage * SMS,
  *
  * \ingroup SMS
  */
-GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo * Info,
+GSM_Error GSM_EncodeMultiPartSMS(GSM_Debug_Info * di,
+				 GSM_MultiPartSMSInfo * Info,
 				 GSM_MultiSMSMessage * SMS);
 
 /**
@@ -963,7 +968,8 @@ GSM_Error GSM_EncodeMultiPartSMS(GSM_MultiPartSMSInfo * Info,
  *
  * \ingroup SMS
  */
-bool GSM_DecodeMultiPartSMS(GSM_MultiPartSMSInfo * Info,
+bool GSM_DecodeMultiPartSMS(GSM_Debug_Info * di,
+			    GSM_MultiPartSMSInfo * Info,
 			    GSM_MultiSMSMessage * SMS, bool ems);
 
 /**
@@ -987,7 +993,8 @@ void GSM_FreeMultiPartSMSInfo(GSM_MultiPartSMSInfo * Info);
  *
  * \ingroup SMS
  */
-GSM_Error GSM_LinkSMS(GSM_MultiSMSMessage ** INPUT,
+GSM_Error GSM_LinkSMS(GSM_Debug_Info * di,
+		      GSM_MultiSMSMessage ** INPUT,
 		      GSM_MultiSMSMessage ** OUTPUT, bool ems);
 
 /**
@@ -1070,7 +1077,7 @@ typedef struct {
  *
  * \ingroup MMS
  */
-GSM_Error GSM_DecodeMMSFileToMultiPart(GSM_File * file,
+GSM_Error GSM_DecodeMMSFileToMultiPart(GSM_Debug_Info * di, GSM_File * file,
 				       GSM_EncodedMultiPartMMSInfo * info);
 
 /**
@@ -1084,6 +1091,7 @@ GSM_Error GSM_ClearMMSMultiPart(GSM_EncodedMultiPartMMSInfo * info);
  * Gets SMS Service Center number and SMS settings.
  *
  * \param s State machine pointer.
+ * \param[in,out] smsc SMSC structure, should contain location.
  *
  * \return Error code.
  *
@@ -1095,6 +1103,7 @@ GSM_Error GSM_GetSMSC(GSM_StateMachine * s, GSM_SMSC * smsc);
  * Sets SMS Service Center number and SMS settings.
  *
  * \param s State machine pointer.
+ * \param[in] smsc SMSC structure.
  *
  * \return Error code.
  *
@@ -1107,6 +1116,7 @@ GSM_Error GSM_SetSMSC(GSM_StateMachine * s, GSM_SMSC * smsc);
  * both SIM and phone).
  *
  * \param s State machine pointer.
+ * \param[out] status Pointer to SMS status structure.
  *
  * \return Error code.
  *
@@ -1118,6 +1128,8 @@ GSM_Error GSM_GetSMSStatus(GSM_StateMachine * s, GSM_SMSMemoryStatus * status);
  * Reads SMS message.
  *
  * \param s State machine pointer.
+ * \param[in,out] sms SMS message data read from phone, location and
+ * folder should be set.
  *
  * \return Error code.
  *
@@ -1133,6 +1145,10 @@ GSM_Error GSM_GetSMS(GSM_StateMachine * s, GSM_MultiSMSMessage * sms);
  * in phone. To do so, you have to call \ref GSM_GetSMS.
  *
  * \param s State machine pointer.
+ * \param[in,out] sms SMS message data read from phone, for subsequent
+ * reads, location and folder might be used by phone driver to determine
+ * reading state.
+ * \param[in] start Whether we start reading from beginning.
  *
  * \return Error code.
  *
@@ -1144,6 +1160,7 @@ GSM_Error GSM_GetNextSMS(GSM_StateMachine * s, GSM_MultiSMSMessage * sms,
  * Sets SMS.
  *
  * \param s State machine pointer.
+ * \param[in] sms SMS message data.
  *
  * \return Error code.
  *
@@ -1155,6 +1172,7 @@ GSM_Error GSM_SetSMS(GSM_StateMachine * s, GSM_SMSMessage * sms);
  * Adds SMS to specified folder.
  *
  * \param s State machine pointer.
+ * \param[in,out] sms SMS message data, location will be updated.
  *
  * \return Error code.
  *
@@ -1166,6 +1184,7 @@ GSM_Error GSM_AddSMS(GSM_StateMachine * s, GSM_SMSMessage * sms);
  * Deletes SMS.
  *
  * \param s State machine pointer.
+ * \param[in] sms SMS structure with SMS location and folder.
  *
  * \return Error code.
  *
@@ -1177,6 +1196,7 @@ GSM_Error GSM_DeleteSMS(GSM_StateMachine * s, GSM_SMSMessage * sms);
  * Sends SMS.
  *
  * \param s State machine pointer.
+ * \param[in] sms SMS structure with SMS data to send.
  *
  * \return Error code.
  *
@@ -1188,6 +1208,8 @@ GSM_Error GSM_SendSMS(GSM_StateMachine * s, GSM_SMSMessage * sms);
  * Sends SMS already saved in phone.
  *
  * \param s State machine pointer.
+ * \param[in] Folder Folder, where message is stored.
+ * \param[in] Location Location, where message is stored.
  *
  * \return Error code.
  *
@@ -1199,6 +1221,7 @@ GSM_Error GSM_SendSavedSMS(GSM_StateMachine * s, int Folder, int Location);
  * Configures fast SMS sending.
  *
  * \param s State machine pointer.
+ * \param[in] enable Whether to enable notifications.
  *
  * \return Error code.
  *
@@ -1210,6 +1233,7 @@ GSM_Error GSM_SetFastSMSSending(GSM_StateMachine * s, bool enable);
  * Enable/disable notification on incoming SMS.
  *
  * \param s State machine pointer.
+ * \param[in] enable Whether to enable notifications.
  *
  * \return Error code.
  *
@@ -1221,6 +1245,7 @@ GSM_Error GSM_SetIncomingSMS(GSM_StateMachine * s, bool enable);
  * Gets network information from phone.
  *
  * \param s State machine pointer.
+ * \param[in] enable Whether to enable notifications.
  *
  * \return Error code.
  *
@@ -1232,6 +1257,8 @@ GSM_Error GSM_SetIncomingCB(GSM_StateMachine * s, bool enable);
  * Returns SMS folders information.
  *
  * \param s State machine pointer.
+ * \param[out] folders folders Pointer to folders structure, which will be
+ * filled in.
  *
  * \return Error code.
  *
@@ -1243,6 +1270,7 @@ GSM_Error GSM_GetSMSFolders(GSM_StateMachine * s, GSM_SMSFolders * folders);
  * Creates SMS folder.
  *
  * \param s State machine pointer.
+ * \param[in] name Name of SMS folder which should be created.
  *
  * \return Error code.
  *
@@ -1254,6 +1282,7 @@ GSM_Error GSM_AddSMSFolder(GSM_StateMachine * s, unsigned char *name);
  * Deletes SMS folder.
  *
  * \param s State machine pointer.
+ * \param[in] ID ID of SMS folder to delete.
  *
  * \return Error code.
  *
@@ -1265,6 +1294,8 @@ GSM_Error GSM_DeleteSMSFolder(GSM_StateMachine * s, int ID);
  * Lists MMS folders.
  *
  * \param s State machine pointer.
+ * \param folders Pointer to structure, whehe folder information will be
+ * stored.
  *
  * \return Error code.
  *
@@ -1276,6 +1307,11 @@ GSM_Error GSM_GetMMSFolders(GSM_StateMachine * s, GSM_MMSFolders * folders);
  * Retrieves next part of MMS file information.
  *
  * \param s State machine pointer.
+ * \param[in,out] FileID File ID will be stored here, might be
+ * used for consequent reads.
+ * \param[in,out] MMSFolder MMS folder ID will be stored here, might be
+ * used for consequent reads.
+ * \param[in] start Whether to start reading.
  *
  * \return Error code.
  *
@@ -1287,6 +1323,7 @@ GSM_Error GSM_GetNextMMSFileInfo(GSM_StateMachine * s, unsigned char *FileID,
  * Activates/deactivates noticing about incoming USSDs (UnStructured Supplementary Services).
  *
  * \param s State machine pointer.
+ * \param[in] enable Whether to enable notifications.
  *
  * \return Error code.
  *
