@@ -12,9 +12,11 @@
 #include <stdio.h>
 #include <string.h>
 
+#include "common.h"
+
 int main(int argc, char **argv)
 {
-	int pos = 0;
+	size_t pos = 0;
 	GSM_ToDoEntry todo;
 	GSM_CalendarEntry cal;
 	GSM_Error error;
@@ -29,7 +31,7 @@ int main(int argc, char **argv)
 
 	/* Configure debugging */
 	debug_info = GSM_GetGlobalDebug();
-	GSM_SetDebugFileDescriptor(stderr, debug_info);
+	GSM_SetDebugFileDescriptor(stderr, false, debug_info);
 	GSM_SetDebugLevel("textall", debug_info);
 
 	/* Check parameters */
@@ -45,17 +47,11 @@ int main(int argc, char **argv)
 
 	/* Open file */
 	f = fopen(argv[1], "r");
-	if (f == NULL) {
-		printf("Could not open %s\n", argv[1]);
-		return 1;
-	}
+	test_result(f != NULL);
 
 	/* Read data */
 	len = fread(buffer, 1, sizeof(buffer) - 1, f);
-	if (!feof(f)) {
-		printf("Could not read whole file %s\n", argv[1]);
-		return 1;
-	}
+	test_result(feof(f));
 
 	/* Zero terminate string */
 	buffer[len] = 0;
@@ -70,16 +66,13 @@ int main(int argc, char **argv)
 
 	/* Parse [iv]Calendar */
 	if (strstr(argv[1], ".ics") != NULL) {
-		error = GSM_DecodeVCALENDAR_VTODO(buffer, &pos, &cal, &todo,
+		error = GSM_DecodeVCALENDAR_VTODO(NULL, buffer, &pos, &cal, &todo,
 				Mozilla_iCalendar, Mozilla_VToDo);
 	} else {
-		error = GSM_DecodeVCALENDAR_VTODO(buffer, &pos, &cal, &todo,
+		error = GSM_DecodeVCALENDAR_VTODO(NULL, buffer, &pos, &cal, &todo,
 				SonyEricsson_VCalendar, SonyEricsson_VToDo);
 	}
-	if (error != ERR_NONE) {
-		printf("Parsing failed: %s\n", GSM_ErrorString(error));
-		return 1;
-	}
+	gammu_test_result(error, "GSM_DecodeVCALENDAR_VTODO");
 
 	/* Generate file if we should */
 	if (generate) {
@@ -97,18 +90,14 @@ int main(int argc, char **argv)
 		} else {
 			backup.ToDo[0] = NULL;
 		}
-		if (GSM_SaveBackupFile(argv[2], &backup, true) != ERR_NONE) {
-			printf("Error saving backup to %s\n", argv[2]);
-			return 1;
-		}
+		error = GSM_SaveBackupFile(argv[2], &backup, true);
+		gammu_test_result(error, "GSM_SaveBackupFile");
 	}
 
 	/* Read file content */
 	GSM_ClearBackup(&backup);
-	if (GSM_ReadBackupFile(argv[2], &backup, GSM_Backup_GammuUCS2) != ERR_NONE) {
-		printf("Error reading backup from %s\n", argv[2]);
-		return 1;
-	}
+	error = GSM_ReadBackupFile(argv[2], &backup, GSM_Backup_GammuUCS2);
+	gammu_test_result(error, "GSM_ReadBackupFile");
 
 	/* Did we read something? */
 	skipcal = false;
@@ -133,6 +122,10 @@ int main(int argc, char **argv)
 	/* Compare content */
 	if (!skipcal) {
 		for (i = 0; i < cal.EntriesNum; i++) {
+			printf("%d %d\n", cal.Entries[i].EntryType,
+				backup.Calendar[0]->Entries[i].EntryType);
+			test_result(cal.Entries[i].EntryType ==
+				backup.Calendar[0]->Entries[i].EntryType);
 			switch (cal.Entries[i].EntryType) {
 				case CAL_TEXT:
 				case CAL_DESCRIPTION:
@@ -144,6 +137,8 @@ int main(int argc, char **argv)
 						backup.Calendar[0]->Entries[i].Text,
 						0)) {
 						printf("Calendar field %d is not the same!\n", i);
+						printf("vc: '%s'\n", DecodeUnicodeConsole(cal.Entries[i].Text));
+						printf("bak: '%s'\n", DecodeUnicodeConsole(backup.Calendar[0]->Entries[i].Text));
 						return 1;
 					}
 					break;
